from flask_scenario_testing.analysis.support import MeasurementsSegment, Measurement


class Segmenter(object):
    def segment(self, measurements: [Measurement]) -> [MeasurementsSegment]:
        pass
